import boto3
import requests
import uuid
from boto3 import Session
from botocore.credentials import RefreshableCredentials
from botocore.session import get_session
from functools import lru_cache
from spackle import log

from spackle.exceptions import SpackleException


class DynamoDB:
    adapter_config = None

    def __init__(self):
        self.client = self._bootstrap_client()

    def get_item(self, key):
        if not self.adapter_config:
            raise SpackleException("Adapter not configured")

        return self.client.get_item(
            TableName=self.adapter_config["table_name"],
            Key={
                "AccountId": {"S": self.adapter_config["identity_id"]},
                **key,
            },
        )

    def query(self, KeyConditionExpression, ExpressionAttributeValues, **kwargs):
        if not self.adapter_config:
            raise SpackleException("Adapter not configured")

        return self.client.query(
            TableName=self.adapter_config["table_name"],
            KeyConditionExpression=f"AccountId = :account_id AND {KeyConditionExpression}",
            ExpressionAttributeValues={
                ":account_id": {"S": self.adapter_config["identity_id"]},
                **ExpressionAttributeValues,
            },
            **kwargs,
        )

    def _bootstrap_client(self):
        log.log_debug("Bootstrapping DynamoDB client...")

        session_credentials = RefreshableCredentials.create_from_metadata(
            metadata=self._refresh_credentials(),
            refresh_using=self._refresh_credentials,
            method="sts-assume-role-with-web-identity",
        )
        session = get_session()
        session._credentials = session_credentials
        autorefresh_session = Session(botocore_session=session)
        return autorefresh_session.client(
            "dynamodb", region_name=self.adapter_config["region"]
        )

    def _refresh_credentials(self):
        log.log_debug("Refreshing credentials...")
        self._configure()
        return self._fetch_credentials()

    def _configure(self):
        from spackle import api_key, api_base

        session = requests.post(
            f"{api_base}/sessions",
            headers={"Authorization": f"Bearer {api_key}"},
        ).json()

        self.adapter_config = session["adapter"]
        log.log_debug("Configured adapter: %s" % session)

        return session

    def _fetch_credentials(self):
        if not self.adapter_config:
            raise SpackleException("Adapter not configured")

        log.log_debug("Assuming aws role %s..." % self.adapter_config["role_arn"])
        sts_client = boto3.client("sts")
        response = sts_client.assume_role_with_web_identity(
            RoleArn=self.adapter_config["role_arn"],
            RoleSessionName=str(uuid.uuid4()),
            WebIdentityToken=self.adapter_config["token"],
        )

        return {
            "access_key": response["Credentials"]["AccessKeyId"],
            "secret_key": response["Credentials"]["SecretAccessKey"],
            "token": response["Credentials"]["SessionToken"],
            "expiry_time": response["Credentials"]["Expiration"].isoformat(),
        }


@lru_cache(maxsize=1)
def get_client():
    log.log_debug("Creating DynamoDB client...")
    return DynamoDB()



import json
import pprint

from spackle import dynamodb, log
from spackle.exceptions import SpackleException

VERSION = 1


class Customer:
    def __init__(self, id, data):
        self.id = id
        self.data = data

    @staticmethod
    def retrieve(customer_id):
        log.log_debug("Retrieving customer data for %s" % customer_id)
        dynamodb_client = dynamodb.get_client()
        response = dynamodb_client.query(
            KeyConditionExpression="CustomerId = :customer_id",
            FilterExpression="Version = :version",
            ExpressionAttributeValues={
                ":customer_id": {"S": customer_id},
                ":version": {"N": str(VERSION)},
            },
            Limit=1,
        )

        items = response.get("Items")
        if not items:
            raise SpackleException("Customer %s not found" % customer_id)

        data = json.loads(items[0]["State"]["S"])
        log.log_debug("Retrieved customer data for %s: %s" % (customer_id, data))
        return Customer(customer_id, data)

    @property
    def features(self):
        return self.data["features"]

    @property
    def flag_features(self):
        return [f for f in self.data["features"] if f["type"] == 0]

    @property
    def limit_features(self):
        return [f for f in self.data["features"] if f["type"] == 1]

    def enabled(self, key):
        for feature in self.flag_features:
            if feature["key"] == key:
                return feature["value_flag"]

        raise SpackleException("Flag feature %s not found" % key)

    def limit(self, key):
        for feature in self.limit_features:
            if feature["key"] == key:
                if feature["value_limit"] is None:
                    return float("inf")
                return feature["value_limit"]

        raise SpackleException("Limit feature %s not found" % key)

    def __repr__(self):
        return f"<Customer ({self.id}):\n{pprint.pformat(self.data)}>"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['spackle']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.26.32,<2.0.0', 'botocore>=1.29.35,<2.0.0', 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'spackle-python',
    'version': '0.0.21',
    'description': '',
    'long_description': '# Spackle Python Library\n\n[![CI](https://github.com/spackleso/spackle-python/actions/workflows/test.yml/badge.svg)](https://github.com/spackleso/spackle-python/actions/workflows/test.yml) [![pypi](https://img.shields.io/pypi/v/spackle-python.svg)](https://pypi.python.org/pypi/spackle-python)\n\nThe Spackle Python library provides optimized access to billing aware flags created on the Spackle platform.\n\n## Documentation\n\nSee the [Python API docs](https://docs.spackle.so/python).\n\n## Setup\n\n### Install the Spackle library\n\n```sh\npip install -U spackle-python\n```\n\n### Configure your environment\nIn order to use Spackle, you need to configure your API key on the `spackle` module. You can find your API key in Spackle app [settings page](https://dashboard.stripe.com/settings/apps/so.spackle.stripe).\n\n```python\nimport spackle\nspackle.api_key = "<api key>"\n```\n\n### Bootstrap the client (optional)\n\nThe Spackle client requires a single initialization step that includes a network request. To front load this process, you can call the `bootstrap` method in your codebase.\n\n```python\nspackle.bootstrap()\n```\n\n## Usage\n\n### Fetch a customer\n\nSpackle uses stripe ids as references to customer features.\n\n```python\ncustomer = spackle.Customer.retrieve("cus_00000000")\n```\n\n### Verify feature access\n\n```python\ncustomer.enabled("feature_key")\n```\n\n### Fetch a feature limit\n\n```python\ncustomer.limit("feature_key")\n```\n\n## Logging\nThe Spackle Python library emits logs as it performs various internal tasks. You can control the verbosity of Spackle\'s logging a few different ways:\n\n1. Set the environment variable SPACKLE_LOG to the value debug or info\n\n   ```sh\n   $ export SPACKLE_LOG=debug\n   ```\n\n2. Set spackle.log:\n\n   ```python\n   import spackle\n   spackle.log = \'debug\'\n   ```\n\n3. Enable it through Python\'s logging module:\n\n   ```python\n   import logging\n   logging.basicConfig()\n   logging.getLogger(\'spackle\').setLevel(logging.DEBUG)\n   ```\n',
    'author': 'Hunter Clarke',
    'author_email': 'hunter@spackle.so',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from spackle.customer import Customer
from spackle.exceptions import SpackleException
from spackle.dynamodb import get_client

api_key = None
api_base = "https://api.spackle.so/v1"


def bootstrap():
    global api_key
    if api_key:
        get_client()
